From e66f29477925a70d335fad757b02cca59d1ea1e2 Mon Sep 17 00:00:00 2001
From: Aaron Patterson <tenderlove@ruby-lang.org>
Date: Wed, 7 Oct 2020 09:28:29 -0700
Subject: [PATCH] Prevent XSS in the Actionable Exceptions middleware

[CVE-2020-8264]
---
 .../action_dispatch/middleware/actionable_exceptions.rb  | 9 ++++++++-
 1 file changed, 8 insertions(+), 1 deletion(-)

diff --git a/actionpack/lib/action_dispatch/middleware/actionable_exceptions.rb b/actionpack/lib/action_dispatch/middleware/actionable_exceptions.rb
index 266fd92ce9..1593ca22d0 100644
--- a/actionpack/lib/action_dispatch/middleware/actionable_exceptions.rb
+++ b/actionpack/lib/action_dispatch/middleware/actionable_exceptions.rb
@@ -1,6 +1,7 @@
 # frozen_string_literal: true
 
 require "erb"
+require "uri"
 require "action_dispatch/http/request"
 require "active_support/actionable_error"
 
@@ -27,7 +28,13 @@ def actionable_request?(request)
       end
 
       def redirect_to(location)
-        body = "<html><body>You are being <a href=\"#{ERB::Util.unwrapped_html_escape(location)}\">redirected</a>.</body></html>"
+        uri = URI.parse location
+
+        if uri.relative? || uri.scheme == "http" || uri.scheme == "https"
+          body = "<html><body>You are being <a href=\"#{ERB::Util.unwrapped_html_escape(location)}\">redirected</a>.</body></html>"
+        else
+          return [400, {"Content-Type" => "text/plain"}, ["Invalid redirection URI"]]
+        end
 
         [302, {
           "Content-Type" => "text/html; charset=#{Response.default_charset}",
-- 
2.26.2

